#!/bin/sh
# the next line restart using pgtclsh \
exec pgtclsh "$0" "$@"

# delete from pg_largeobject orphan loids
# it is very similar to vacuumlo but works on
# type 'lo' without requiring cast functions into
# the database

# ATTENTION:
# DO NOT USE ON DATABASES HAVING OID LO DATA TYPE !!!


proc query_exec {dbname query {risp {}}} {

# risp fa ritornare l'handle del risultato della query
# in modo da poter accedere ai risultati richiesti
# (in tal caso non si ripulisce con pg_result -clear)
# risp e' il terzo argomento e puo' essere una stringa qualsiasi (es. -risp)
# Es di utilizzo con risp: 
#       set query "select count(*) as tupnum from tablename"
#       set qry_handle [query_exec dbname $query -return_handle]
#       pg_result $qry_handle -assign risp
#       set acc $risp(0,tupnum)
#       pg_result $qry_handle -clear

# solo con 2 argomenti viene ritornato solo lo stato finale
# e non occorre ripulire con pg_result -clear
# Es:
#       set query "select count(*) as tupnum from tablename"
#       set ris [query_exec $dbname $query]

global lconn

        set dbhandle $lconn
        #puts $lfout "[clock clicks] ($dbname) $query..."
        set res [pg_exec $dbhandle $query]
        set ris [pg_result $res -status]
        set err [pg_result $res -error]
        if {[string length $err]!=0} {
                #puts $lfout "[clock clicks] ($dbname) ...$ris - $err"
                pg_result $res -clear
                return -code error "$err"
                } else {
                #puts $lfout "[clock clicks] ($dbname) ...$ris"
                if {[string length $risp]!=0} {
                        return $res
                        } else {
                        pg_result $res -clear
                        return $ris
                        }
                }
        
        }




################
#  M A I N
################


global lconn
set VERSION 1.1.0
puts "\n\avacuumlo.tcl  $VERSION\n"

if { ($argc!=1) }  {
        puts stdout [format "\a\nUsage: vacuumlo.tcl  <dbname> \n"]
        return -1
        }

set dbname [lindex $argv 0]
set port 5432
set lfout stdout
set lhost localhost
set ldbfout stdout
set lofout stdout


puts "ATTENTION please:"
puts " "
puts " This utility works only with 'lo' large object data type"
puts " "
puts " Running this utility on database having different" 
puts " large object data type (such as 'oid') leads to"
puts " large object DELETING from pg_largeobject!!!" 
puts " "
puts [format "\a\nDo you want to continue? (Y/N)"]
#puts "Do you want to continue? (Y/N)"
gets stdin risp
if {[string compare [string toupper "$risp"] "Y"]!=0} {
	puts " "
	puts "No change to $dbname database has been made."
	puts " "
	exit
}


if {[catch {pg_connect $dbname -port $port -host $lhost} lconn]} {
	puts "[clock format [clock second] -format %T] trying to connect to database $dbname on $lhost (port $port):\n$lconn"
	exit
	}

set now [clock format [clock second] -format %T]


# search for ALL user tables with lo type fields
set query "select c.relname as table, a.attname as lofield from pg_class c, pg_attribute a, pg_type t where a.attrelid=c.oid  and a.atttypid=t.oid and  t.typname=\'lo\' and a.attnum>0 and c.relname not like \'pg\\_%\'"
catch {unset listlofields}
pg_select $lconn $query lofields {
        lappend listlofields($lofields(table)) $lofields(lofield)
        }



if {![info exists listlofields] } {
	puts "\n\a[clock format [clock second] -format %T] No attributes of type LO founded in $dbname!\n"
	pg_disconnect $lconn
	exit
	}



# drop/create vacuum_lo table
set query "drop table pgr_vacuumlo"
puts $lofout  "$now [clock clicks] $query..."
if {[catch {query_exec $dbname $query} ret]} {
        puts $lofout  "$now [clock clicks] ...$ret"
        } else {
        puts $lofout  "$now [clock clicks] ...$ret"
        }


# populating pgr_loids with distinct pg_largeobject loid 
set query "select distinct loid into pgr_vacuumlo from pg_largeobject"
puts $lofout  "$now [clock clicks] $query..."
if {[catch {query_exec $dbname $query} ret]} {
        puts $ldbfout  "$now [clock clicks] ...$ret"
	pg_disconnect $lconn
        return -code error "$ret"
        } else {
        puts $lofout  "$now [clock clicks] ...$ret"
        }


# creating index on pgr_vacuumlo
puts -nonewline $lofout "$now [clock clicks] creating index on pgr_vacuumlo..."
query_exec $dbname "create index pgr_vacuumlo_idx on pgr_vacuumlo (loid)"
puts $lofout " Done"


# scan tables with lo fields deleting LO value from pgr_vacuumlo
# so at the end of this process pgr_vacuumlo will get 
# the orphan loids
puts $ldbfout  "$now [clock clicks] scan tables with LO fields deleting LO value from pgr_vacuumlo..."
foreach table [array names listlofields] {

        # table lofields
        set lofields  $listlofields($table)
        set lofieldsString [join $lofields ", "]
        puts $lofout "$now [clock clicks] working on $table: $lofieldsString"
        
        set query "select $lofieldsString from $table"
        pg_select $lconn $query row {
                foreach lofield $lofields {
                        if {[string compare $row($lofield) ""]!=0} {
                                query_exec $dbname "delete from pgr_vacuumlo where loid=$row($lofield)"
                                }
                        };# end of foreach

                };# end of pg_select 

        };# end of foreach



# deleting from pg_largeobject where loid in pgr_vacuumlo
set query "delete from pg_largeobject where loid in (select loid from pgr_vacuumlo)"
puts $lofout  "$now [clock clicks] $query..."
if {[catch {query_exec $dbname $query} ret]} {
        puts $ldbfout  "$now [clock clicks] ...$ret"
        puts $lofout  "$now [clock clicks] ...$ret"
	pg_disconnect $lconn
        return -code error "$ret"
        } else {
        puts $lofout  "$now [clock clicks] ...$ret"
        }


# vacuuming pg_largeobject
set query "vacuum analyze pg_largeobject"
puts $lofout  "$now [clock clicks] $query..."
set ris [query_exec $dbname $query]
puts $lofout  "$now [clock clicks] $ris"

set now [clock format [clock second] -format %T]
puts $ldbfout [format "\n$now [clock clicks] vacuumlo \t$dbname Done!\n"]


pg_disconnect $lconn
exit


